"Test functions."

# For support of python 2.5
from __future__ import with_statement

import numpy as np
from numpy.testing import (assert_equal, assert_array_equal,
                           assert_array_almost_equal)
nan = np.nan
import bottlechest as bn


def arrays(dtypes=bn.dtypes):
    "Iterator that yield arrays to use for unit testing."
    ss = {}
    ss[0] = {'size':  0, 'shapes': [(0,), (0,0), (2,0), (2,0,1)]}
    ss[1] = {'size':  4, 'shapes': [(4,)]}
    ss[2] = {'size':  6, 'shapes': [(1,6), (2,3)]}
    ss[3] = {'size':  6, 'shapes': [(1,2,3)]}
    ss[4] = {'size': 24, 'shapes': [(1,2,3,4)]}  # Unaccelerated
    for ndim in ss:
        size = ss[ndim]['size']
        shapes = ss[ndim]['shapes']
        for dtype in dtypes:
            a = np.arange(size, dtype=dtype)
            for shape in shapes:
                a = a.reshape(shape)
                yield a
                yield -a
            for i in range(0, a.size, 2):
                a.flat[i] *= -1
                yield a

def unit_maker(func, func0):
    "Test bn.(arg)partsort gives same output as bn.slow.(arg)partsort."
    msg = '\nfunc %s | input %s (%s) | shape %s | n %d | axis %s\n'
    msg += '\nInput array:\n%s\n'
    for i, arr in enumerate(arrays()):
        for axis in list(range(-arr.ndim, arr.ndim)) + [None]:
            if axis is None:
                n = arr.size
            else:
                n = arr.shape[axis]
            n = max(n // 2, 1)
            with np.errstate(invalid='ignore'):
                actual = func(arr.copy(), n, axis=axis)
                actual[:n] = np.sort(actual[:n], axis=axis)
                actual[n:] = np.sort(actual[n:], axis=axis)
                desired = func0(arr.copy(), n, axis=axis)
                if 'arg' in func.__name__:
                    desired[:n] = np.sort(desired[:n], axis=axis)
                    desired[n:] = np.sort(desired[n:], axis=axis)
            tup = (func.__name__, 'a'+str(i), str(arr.dtype),
                   str(arr.shape), n, str(axis), arr)
            err_msg = msg % tup
            assert_array_equal(actual, desired, err_msg)
            err_msg += '\n dtype mismatch %s %s'
            if hasattr(actual, 'dtype') or hasattr(desired, 'dtype'):
                da = actual.dtype
                dd = desired.dtype
                assert_equal(da, dd, err_msg % (da, dd))

def test_partsort():
    "Test partsort."
    yield unit_maker, bn.partsort, bn.slow.partsort

def test_argpartsort():
    "Test argpartsort."
    yield unit_maker, bn.argpartsort, bn.slow.argpartsort



import unittest
import sys
import bottlechest as bn
import numpy as np
import scipy.sparse as sp

class TestContingency(unittest.TestCase):
    def test_1d_int(self):
        data = np.array([0, 1, 1, 2, 1])
        bb = [0, 1, 1, 0, 0]
        for b in [bb, np.array(bb, dtype=np.int8), np.array(bb, dtype=float)]:
            counts, nans = bn.contingency(data, b, 2, 1)
            np.testing.assert_almost_equal(counts, [[1, 1, 1], [0, 2, 0]])
            np.testing.assert_almost_equal(nans, np.zeros(2))

    def test_1d_float(self):
        nan = float("nan")
        data = np.array([0, 1, nan, 2, 1], dtype=float)
        bb = [0, 1, 1, 0, 0]
        for b in [bb, np.array(bb, dtype=np.int8), np.array(bb, dtype=float)]:
            counts, nans = bn.contingency(data, b, 2, 1)
            np.testing.assert_almost_equal(counts, [[1, 1, 1], [0, 1, 0]])
            np.testing.assert_almost_equal(nans, [0, 1])

    def test_1d_mask_int(self):
        data = np.array([0, 1, 1, 2, 1])
        bb = [0, 1, 1, 0, 0]
        counts, nans = bn.contingency(data, bb, 2, 1, mask=[1])
        np.testing.assert_almost_equal(counts, [[1, 1, 1], [0, 2, 0]])
        np.testing.assert_almost_equal(nans, [0, 0])

        counts, nans = bn.contingency(data, bb, 2, 1, mask=[0])
        np.testing.assert_almost_equal(counts, np.zeros((2, 3)))
        np.testing.assert_almost_equal(nans, [0, 0])

    def test_1d_mask_float(self):
        nan = float("nan")
        data = np.array([0, 1, nan, 2, 1], dtype=float)
        bb = [0, 1, 1, 0, 0]
        counts, nans = bn.contingency(data, bb, 2, 1, mask=[1])
        np.testing.assert_almost_equal(counts, [[1, 1, 1], [0, 1, 0]])
        np.testing.assert_almost_equal(nans, [0, 1])

        counts, nans = bn.contingency(data, bb, 2, 1, mask=[0])
        np.testing.assert_almost_equal(counts, np.zeros((2, 3)))
        np.testing.assert_almost_equal(nans, [0, 0])

    def test_1d_weighted_int(self):
        data = np.array([0, 1, 1, 2, 1])
        bb = [0, 1, 1, 0, 0]
        counts, nans = bn.contingency(data, bb, 2, 1, weights=[1, 2, 3, 4, 5])
        np.testing.assert_almost_equal(counts, [[1, 5, 3], [0, 3, 0]])
        np.testing.assert_almost_equal(nans, np.zeros(2))

    def test_1d_weighted_int(self):
        nan = float("nan")
        data = np.array([0, 1, nan, 2, 1], dtype=float)
        bb = [0, 1, 1, 0, 0]
        for b in [bb, np.array(bb, dtype=np.int8), np.array(bb, dtype=float)]:
            counts, nans = bn.contingency(data, b, 2, 1, weights=[1, 2, 3, 4, 5])
            np.testing.assert_almost_equal(counts, [[1, 5, 4], [0, 2, 0]])
            np.testing.assert_almost_equal(nans, [0, 3])

    def test_simple_int(self):
        data = np.array([[0, 1, 1, 2, 1],
                         [1, 1, 1, 0, 1],
                         [0, 0, 3, 0, 0]], dtype=int)
        for b in [
                np.array([1, 0, 1], dtype=np.int8),
                np.array([1, 0, 1], dtype=float),
                [1, 0, 1]]:
            counts, nans = bn.contingency(data, b, 3, 1)
            np.testing.assert_almost_equal(counts[0], [[0, 1, 0, 0], [2, 0, 0, 0]])
            np.testing.assert_almost_equal(counts[1], [[0, 1, 0, 0], [1, 1, 0, 0]])
            np.testing.assert_almost_equal(counts[2], [[0, 1, 0, 0], [0, 1, 0, 1]])
            np.testing.assert_almost_equal(counts[3], [[1, 0, 0, 0], [1, 0, 1, 0]])
            np.testing.assert_almost_equal(counts[4], [[0, 1, 0, 0], [1, 1, 0, 0]])
            np.testing.assert_almost_equal(nans, np.zeros((5, 2)))


    def test_simple_float(self):
        nan = float("nan")
        data = np.array([[0, 1, 1, 2, 1],
                         [1, 1, 1, nan, 1],
                         [0, 0, 3, nan, nan]], dtype=float)

        counts, nans = bn.contingency(data, [1, 0, 1], 3, 1)
        np.testing.assert_almost_equal(counts[0], [[0, 1, 0, 0], [2, 0, 0, 0]])
        np.testing.assert_almost_equal(counts[1], [[0, 1, 0, 0], [1, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[2], [[0, 1, 0, 0], [0, 1, 0, 1]])
        np.testing.assert_almost_equal(counts[3], [[0, 0, 0, 0], [0, 0, 1, 0]])
        np.testing.assert_almost_equal(counts[4], [[0, 1, 0, 0], [0, 1, 0, 0]])
        np.testing.assert_almost_equal(nans, [[0, 0], [0, 0], [0, 0], [1, 1], [0, 1]])

    def test_weighted_int(self):
        data = np.array([[0, 1, 1, 2, 1],
                         [1, 1, 1, 0, 1],
                         [0, 0, 3, 0, 0]], dtype=int)
        counts, nans = bn.contingency(data, [1, 0, 1], 3, 1, weights=[1, 2, 3])
        np.testing.assert_almost_equal(counts[0], [[0, 2, 0, 0], [4, 0, 0, 0]])
        np.testing.assert_almost_equal(counts[1], [[0, 2, 0, 0], [3, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[2], [[0, 2, 0, 0], [0, 1, 0, 3]])
        np.testing.assert_almost_equal(counts[3], [[2, 0, 0, 0], [3, 0, 1, 0]])
        np.testing.assert_almost_equal(counts[4], [[0, 2, 0, 0], [3, 1, 0, 0]])
        np.testing.assert_almost_equal(nans, np.zeros((5, 2)))


    def test_weighted_float(self):
        nan = float("nan")
        data = np.array([[0, 1, 1, 2, 1],
                         [1, 1, 1, nan, 1],
                         [0, 0, 3, nan, nan]], dtype=float)

        counts, nans = bn.contingency(data, [1, 0, 1], 3, 1, weights=[1, 2, 3])
        np.testing.assert_almost_equal(counts[0], [[0, 2, 0, 0], [4, 0, 0, 0]])
        np.testing.assert_almost_equal(counts[1], [[0, 2, 0, 0], [3, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[2], [[0, 2, 0, 0], [0, 1, 0, 3]])
        np.testing.assert_almost_equal(counts[3], [[0, 0, 0, 0], [0, 0, 1, 0]])
        np.testing.assert_almost_equal(counts[4], [[0, 2, 0, 0], [0, 1, 0, 0]])
        np.testing.assert_almost_equal(nans, [[0, 0], [0, 0], [0, 0], [2, 3], [0, 3]])


    def test_mask_int(self):
        data = np.array([[0, 1, 1, 2, 1],
                         [1, 1, 1, 0, 1],
                         [0, 0, 3, 0, 0]], dtype=int)
        for b in [
                np.array([1, 0, 1], dtype=np.int8),
                np.array([1, 0, 1], dtype=float),
                [1, 0, 1]]:
            counts, nans = bn.contingency(data, b, 3, 1, mask=[1, 1, 0, 0, 1])
            np.testing.assert_almost_equal(counts[0], [[0, 1, 0, 0], [2, 0, 0, 0]])
            np.testing.assert_almost_equal(counts[1], [[0, 1, 0, 0], [1, 1, 0, 0]])
            np.testing.assert_almost_equal(counts[2], np.zeros((2, 4)))
            np.testing.assert_almost_equal(counts[3], np.zeros((2, 4)))
            np.testing.assert_almost_equal(counts[4], [[0, 1, 0, 0], [1, 1, 0, 0]])
            np.testing.assert_almost_equal(nans, np.zeros((5, 2)))


    def test_mask_float(self):
        nan = float("nan")
        data = np.array([[0, 1, 1, 2, 1],
                         [1, 1, 1, nan, 1],
                         [0, 0, 3, nan, nan]], dtype=float)

        counts, nans = bn.contingency(data, [1, 0, 1], 3, 1, mask=[1, 1, 0, 0, 1])
        np.testing.assert_almost_equal(counts[0], [[0, 1, 0, 0], [2, 0, 0, 0]])
        np.testing.assert_almost_equal(counts[1], [[0, 1, 0, 0], [1, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[2], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[3], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[4], [[0, 1, 0, 0], [0, 1, 0, 0]])
        np.testing.assert_almost_equal(nans, [[0, 0], [0, 0], [0, 0], [0, 0], [0, 1]])

    def test_mask_weighted_int(self):
        data = np.array([[0, 1, 1, 2, 1],
                         [1, 1, 1, 0, 1],
                         [0, 0, 3, 0, 0]], dtype=int)
        counts, nans = bn.contingency(data, [1, 0, 1], 3, 1,
                                      weights=[1, 2, 3], mask=[1, 1, 0, 0, 1])
        np.testing.assert_almost_equal(counts[0], [[0, 2, 0, 0], [4, 0, 0, 0]])
        np.testing.assert_almost_equal(counts[1], [[0, 2, 0, 0], [3, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[2], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[3], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[4], [[0, 2, 0, 0], [3, 1, 0, 0]])
        np.testing.assert_almost_equal(nans, np.zeros((5, 2)))


    def test_mask_weighted_float(self):
        nan = float("nan")
        data = np.array([[0, 1, 1, 2, 1],
                         [1, 1, 1, nan, 1],
                         [0, 0, 3, nan, nan]], dtype=float)

        counts, nans = bn.contingency(data, [1, 0, 1], 3, 1,
                                      weights=[1, 2, 3], mask=[1, 1, 0, 0, 1])
        np.testing.assert_almost_equal(counts[0], [[0, 2, 0, 0], [4, 0, 0, 0]])
        np.testing.assert_almost_equal(counts[1], [[0, 2, 0, 0], [3, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[2], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[3], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[4], [[0, 2, 0, 0], [0, 1, 0, 0]])
        np.testing.assert_almost_equal(nans, [[0, 0], [0, 0], [0, 0], [0, 0], [0, 3]])



    def test_sparse_int(self):
        data = np.array([1, 1, 2, 2, 1, 3])
        indptr = [0, 3, 4, 6]
        indices = [0, 1, 2, 0, 1, 2]
        a = sp.csr_matrix((data, indices, indptr), shape=(3, 4))
        counts, nans = bn.contingency(a, [1, 0, 1], 3, 1)
        np.testing.assert_almost_equal(counts[0], [[0, 0, 1, 0], [0, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[1], [[0, 0, 0, 0], [0, 2, 0, 0]])
        np.testing.assert_almost_equal(counts[2], [[0, 0, 0, 0], [0, 0, 1, 1]])
        np.testing.assert_almost_equal(counts[3], np.zeros((2, 4)))
        np.testing.assert_almost_equal(nans, np.zeros((4, 2)))

    def test_sparse_float(self):
        data = np.array([1, 1, 2, 2, 1, 3], dtype=float)
        indptr = [0, 3, 4, 6]
        indices = [0, 1, 2, 0, 1, 2]
        a = sp.csr_matrix((data, indices, indptr), shape=(3, 4))
        counts, nans = bn.contingency(a, [1, 0, 1], 3, 1)
        np.testing.assert_almost_equal(counts[0], [[0, 0, 1, 0], [0, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[1], [[0, 0, 0, 0], [0, 2, 0, 0]])
        np.testing.assert_almost_equal(counts[2], [[0, 0, 0, 0], [0, 0, 1, 1]])
        np.testing.assert_almost_equal(counts[3], np.zeros((2, 4)))
        np.testing.assert_almost_equal(nans, np.zeros((4, 2)))

    def test_sparse_weight_int(self):
        data = np.array([1, 1, 2, 2, 1, 3])
        indptr = [0, 3, 4, 6]
        indices = [0, 1, 2, 0, 1, 2]
        a = sp.csr_matrix((data, indices, indptr), shape=(3, 4))
        counts, nans = bn.contingency(a, [1, 0, 1], 3, 1, weights=[1, 2, 3])
        np.testing.assert_almost_equal(counts[0], [[0, 0, 2, 0], [0, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[1], [[0, 0, 0, 0], [0, 4, 0, 0]])
        np.testing.assert_almost_equal(counts[2], [[0, 0, 0, 0], [0, 0, 1, 3]])
        np.testing.assert_almost_equal(counts[3], np.zeros((2, 4)))

    def test_sparse_weight_float(self):
        data = np.array([1, 1, 2, 2, 1, 3], dtype=float)
        indptr = [0, 3, 4, 6]
        indices = [0, 1, 2, 0, 1, 2]
        a = sp.csr_matrix((data, indices, indptr), shape=(3, 4))
        counts, nans = bn.contingency(a, [1, 0, 1], 3, 1, weights=[1, 2, 3])
        np.testing.assert_almost_equal(counts[0], [[0, 0, 2, 0], [0, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[1], [[0, 0, 0, 0], [0, 4, 0, 0]])
        np.testing.assert_almost_equal(counts[2], [[0, 0, 0, 0], [0, 0, 1, 3]])
        np.testing.assert_almost_equal(counts[3], np.zeros((2, 4)))

    def test_sparse_mask_int(self):
        data = np.array([1, 1, 2, 2, 1, 3])
        indptr = [0, 3, 4, 6]
        indices = [0, 1, 2, 0, 1, 2]
        a = sp.csr_matrix((data, indices, indptr), shape=(3, 4))
        counts, nans = bn.contingency(a, [1, 0, 1], 3, 1, mask=[1, 0, 0, 1])
        np.testing.assert_almost_equal(counts[0], [[0, 0, 1, 0], [0, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[1], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[2], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[3], np.zeros((2, 4)))

    def test_sparse_mask_float(self):
        data = np.array([1, 1, 2, 2, 1, 3], dtype=float)
        indptr = [0, 3, 4, 6]
        indices = [0, 1, 2, 0, 1, 2]
        a = sp.csr_matrix((data, indices, indptr), shape=(3, 4))
        counts, nans = bn.contingency(a, [1, 0, 1], 3, 1, mask=[1, 0, 0, 1])
        np.testing.assert_almost_equal(counts[0], [[0, 0, 1, 0], [0, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[1], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[2], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[3], np.zeros((2, 4)))


    def test_sparse_mask_weight_int(self):
        data = np.array([1, 1, 2, 2, 1, 3])
        indptr = [0, 3, 4, 6]
        indices = [0, 1, 2, 0, 1, 2]
        a = sp.csr_matrix((data, indices, indptr), shape=(3, 4))
        counts, nans = bn.contingency(a, [1, 0, 1], 3, 1,
                                      weights=[1, 2, 3], mask=[1, 0, 0, 1])
        np.testing.assert_almost_equal(counts[0], [[0, 0, 2, 0], [0, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[1], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[2], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[3], np.zeros((2, 4)))

    def test_sparse_mask_weight_float(self):
        data = np.array([1, 1, 2, 2, 1, 3], dtype=float)
        indptr = [0, 3, 4, 6]
        indices = [0, 1, 2, 0, 1, 2]
        a = sp.csr_matrix((data, indices, indptr), shape=(3, 4))
        counts, nans = bn.contingency(a, [1, 0, 1], 3, 1,
                                      weights=[1, 2, 3], mask=[1, 0, 0, 1])
        np.testing.assert_almost_equal(counts[0], [[0, 0, 2, 0], [0, 1, 0, 0]])
        np.testing.assert_almost_equal(counts[1], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[2], np.zeros((2, 4)))
        np.testing.assert_almost_equal(counts[3], np.zeros((2, 4)))

if __name__ == "__main__":
    unittest.main()



#for backward compatibility

from .ss import *
from .rankdata import *
from .median import *
from .allnan import *
from .anynan import *
from .argpartsort import *
from .bincount import *
from .contingency import *
from .countnans import *
from .nanargmax import *
from .nanargmin import *
from .nanequal import *
from .nanmax import *
from .nanmean import *
from .nanmin import *
from .nanstd import *
from .nansum import *
from .nanvar import *
from .nn import *
from .partsort import *
from .rankdata import *
from .replace import *
from .ss import *
from .stats import *
from .valuecount import *



"nansum template"

from copy import deepcopy
import bottlechest as bn

__all__ = ["nansum"]

FLOAT_DTYPES = [x for x in bn.dtypes if 'float' in x]
INT_DTYPES = [x for x in bn.dtypes if 'int' in x]


# Float dtypes (not axis=None) ----------------------------------------------

floats = {}
floats['dtypes'] = FLOAT_DTYPES
floats['axisNone'] = False
floats['force_output_dtype'] = False
floats['reuse_non_nan_func'] = False

floats['top'] = """
@cython.boundscheck(False)
@cython.wraparound(False)
def NAME_NDIMd_DTYPE_axisAXIS(np.ndarray[np.DTYPE_t, ndim=NDIM] a):
    "Sum of NDIMd array with dtype=DTYPE along axis=AXIS ignoring NaNs."
    cdef int allnan = 1
    cdef np.DTYPE_t asum = 0, ai
"""

loop = {}
loop[2] = """\
    for iINDEX0 in range(nINDEX0):
        asum = 0
        allnan = 1
        for iINDEX1 in range(nINDEX1):
            ai = a[INDEXALL]
            if ai == ai:
                asum += ai
                allnan = 0
        if allnan == 0:       
            y[INDEXPOP] = asum
        else:
            y[INDEXPOP] = NAN
    return y
"""
floats['loop'] = loop

# Float dtypes (axis=None) --------------------------------------------------

floats_None = deepcopy(floats)
floats_None['axisNone'] = True

returns = """\
    if allnan == 0:
        return np.DTYPE(asum)
    else:
        return np.DTYPE(NAN)
"""        

loop = {}
loop[1] = """\
    for iINDEX0 in range(nINDEX0):
        ai = a[INDEXALL]
        if ai == ai:
            asum += ai
            allnan = 0
""" + returns
loop[2] = """\
    for iINDEX0 in range(nINDEX0):
        for iINDEX1 in range(nINDEX1):
            ai = a[INDEXALL]
            if ai == ai:
                asum += ai
                allnan = 0
""" + returns
floats_None['loop'] = loop

# Int dtypes (not axis=None) ------------------------------------------------

ints = deepcopy(floats)
ints['dtypes'] = INT_DTYPES 

ints['top'] = """
@cython.boundscheck(False)
@cython.wraparound(False)
def NAME_NDIMd_DTYPE_axisAXIS(np.ndarray[np.DTYPE_t, ndim=NDIM] a):
    "Sum of NDIMd array with dtype=DTYPE along axis=AXIS ignoring NaNs."
    cdef np.DTYPE_t asum = 0, ai
"""

loop = {}
loop[2] = """\
    for iINDEX0 in range(nINDEX0):
        asum = 0
        for iINDEX1 in range(nINDEX1):
            asum += a[INDEXALL]
        y[INDEXPOP] = asum
    return y
"""
ints['loop'] = loop

# Int dtypes (axis=None) ----------------------------------------------------

ints_None = deepcopy(ints) 
ints_None['top'] = ints['top'] + "    cdef Py_ssize_t size\n"
ints_None['axisNone'] = True

loop = {}
loop[1] = """\
    size = nINDEX0
    for iINDEX0 in range(nINDEX0):
        asum += a[INDEXALL]
    return np.DTYPE(asum)
"""
loop[2] = """\
    size = nINDEX0 * nINDEX1    
    for iINDEX0 in range(nINDEX0):
        for iINDEX1 in range(nINDEX1):
            asum += a[INDEXALL]
    return np.DTYPE(asum)
"""
ints_None['loop'] = loop

# Slow, unaccelerated ndim/dtype --------------------------------------------

slow = {}
slow['name'] = "nansum"
slow['signature'] = "arr"
slow['func'] = "bn.slow.nansum(arr, axis=AXIS)"

# Template ------------------------------------------------------------------

nansum = {}
nansum['name'] = 'nansum'
nansum['is_reducing_function'] = True
nansum['cdef_output'] = True
nansum['slow'] = slow
nansum['templates'] = {}
nansum['templates']['float'] = floats
nansum['templates']['float_None'] = floats_None
nansum['templates']['int'] = ints
nansum['templates']['int_None'] = ints_None
nansum['pyx_file'] = 'func/%sbit/nansum.pyx'

nansum['main'] = '''"nansum auto-generated from template"

def nansum(arr, axis=None):
    """
    Sum of array elements along given axis ignoring NaNs.

    When the input has an integer type with less precision than the default
    platform integer, the default platform integer is used for the
    accumulator and return values.

    Parameters
    ----------
    arr : array_like
        Array containing numbers whose sum is desired. If `arr` is not an
        array, a conversion is attempted.
    axis : {int, None}, optional
        Axis along which the sum is computed. The default (axis=None) is to
        compute the sum of the flattened array.

    Returns
    -------
    y : ndarray
        An array with the same shape as `arr`, with the specified axis removed.
        If `arr` is a 0-d array, or if axis is None, a scalar is returned. 
    
    Notes
    -----
    No error is raised on overflow.

    If positive or negative infinity are present the result is positive or
    negative infinity. But if both positive and negative infinity are present,
    the result is Not A Number (NaN).

    Examples
    --------
    >>> bn.nansum(1)
    1
    >>> bn.nansum([1])
    1
    >>> bn.nansum([1, np.nan])
    1.0
    >>> a = np.array([[1, 1], [1, np.nan]])
    >>> bn.nansum(a)
    3.0
    >>> bn.nansum(a, axis=0)
    array([ 2.,  1.])

    When positive infinity and negative infinity are present:

    >>> bn.nansum([1, np.nan, np.inf])
    inf
    >>> bn.nansum([1, np.nan, np.NINF])
    -inf
    >>> bn.nansum([1, np.nan, np.inf, np.NINF])
    nan
    
    """
    func, arr = nansum_selector(arr, axis)
    return func(arr)

def nansum_selector(arr, axis):
    """
    Return nansum function and array that matches `arr` and `axis`.
    
    Under the hood Bottleneck uses a separate Cython function for each
    combination of ndim, dtype, and axis. A lot of the overhead in
    bn.nansum() is in checking that `axis` is within range, converting `arr`
    into an array (if it is not already an array), and selecting the function
    to use to calculate the sum.

    You can get rid of the overhead by doing all this before you, for example,
    enter an inner loop, by using this function.

    Parameters
    ----------
    arr : array_like
        Input array. If `arr` is not an array, a conversion is attempted.
    axis : {int, None}
        Axis along which the sum is to be computed.
    
    Returns
    -------
    func : function
        The nansum function that matches the number of dimensions and dtype
        of the input array and the axis along which you wish to find the sum.
    a : ndarray
        If the input array `arr` is not a ndarray, then `a` will contain the
        result of converting `arr` into a ndarray.

    Examples
    --------
    Create a numpy array:

    >>> arr = np.array([1.0, np.nan, 3.0])
    
    Obtain the function needed to determine the nansum of `arr` along axis=0:

    >>> func, a = bn.func.nansum_selector(arr, axis=0)
    >>> func
    <function nansum_1d_float64_axis0>
    
    Use the returned function and array to determine the sum:

    >>> func(a)
    4.0

    """
    cdef np.ndarray a
    if type(arr) is np.ndarray:
        a = arr
    else:    
        a = np.array(arr, copy=False)
    cdef int ndim = PyArray_NDIM(a)
    cdef int dtype = PyArray_TYPE(a)
    if dtype < NPY_int_:
        a = a.astype(np.int_)
        dtype = PyArray_TYPE(a)
    if (axis is not None) and (axis < 0):
        axis += ndim
    cdef tuple key = (ndim, dtype, axis)
    try:
        func = nansum_dict[key]
    except KeyError:
        if axis is not None:
            if (axis < 0) or (axis >= ndim):
                raise ValueError("axis(=%d) out of bounds" % axis)
        try:
            func = nansum_slow_dict[axis]
        except KeyError:
            tup = (str(ndim), str(a.dtype), str(axis))
            raise TypeError("Unsupported ndim/dtype/axis (%s/%s/%s)." % tup)
    return func, a
'''   



"""
This file is only Used to build the .so files with "make build".

This setup.py is NOT used to install the Bottlechest package. The Bottlechest
setup.py file is bottlechest/setup.py
"""

import os
import os.path
from distutils.core import setup
from distutils.extension import Extension
import numpy as np

# Is the default numpy int 32 or 64 bits?
if np.int_ == np.int32:
    bits = '32'
elif np.int_ == np.int64:
    bits = '64'
else:
    raise ValueError("Your OS does not appear to be 32 or 64 bits.")


cfiles = [ a[:-2] for a in os.listdir("bottlechest/src/func/%sbit/" % bits) \
    if a.endswith(".c") ]

extensions = [ Extension(cf,
               sources=["bottlechest/src/func/%sbit/%s.c" % (bits, cf)],
               include_dirs=[np.get_include()]) for cf in cfiles ]

setup(
  name = 'func',
  ext_package= "bottlechest",
  ext_modules = extensions
)




import unittest
import sys
import bottlechest as bn
import numpy as np


class TestCountNans(unittest.TestCase):
    def test2d(self):
        a = np.ones((4, 5))
        for x, y in ((0, 0), (0, 1), (0, 2), (3, 1)):
            a[x, y] = float("nan")
        self.assertEqual(bn.countnans(a), 4)
        self.assertEqual(bn.slow.countnans(a), 4)
        np.testing.assert_array_equal(bn.countnans(a, axis=0), [1, 2, 1, 0, 0])
        np.testing.assert_array_equal(bn.slow.countnans(a, axis=0), [1, 2, 1, 0, 0])
        np.testing.assert_array_equal(bn.countnans(a, axis=1), [3, 0, 0, 1])
        np.testing.assert_array_equal(bn.slow.countnans(a, axis=1), [3, 0, 0, 1])

    def test2d_w(self):
        a = np.ones((4, 5))
        w = np.random.random((4, 5))
        for x, y in ((0, 0), (0, 1), (0, 2), (3, 1)):
            a[x, y] = float("nan")
        self.assertAlmostEqual(bn.countnans(a, weights=w),
                               bn.slow.countnans(a, weights=w))
        np.testing.assert_almost_equal(bn.countnans(a, weights=[0.1, 0.2, 0.3, 0.4], axis=0), [0.1, 0.5, 0.1, 0, 0])
        np.testing.assert_almost_equal(bn.slow.countnans(a, weights=[0.1, 0.2, 0.3, 0.4], axis=0), [0.1, 0.5, 0.1, 0, 0])
        np.testing.assert_almost_equal(bn.countnans(a, weights=[0.1, 0.2, 0.3, 0.4, 0.5], axis=1), [0.6, 0, 0, 0.2])
        np.testing.assert_almost_equal(bn.slow.countnans(a, weights=[0.1, 0.2, 0.3, 0.4, 0.5], axis=1), [0.6, 0, 0, 0.2])

    def test1d(self):
        a = np.ones(5)
        a[2] = a[4] = float("nan")
        self.assertEqual(bn.countnans(a), 2)
        self.assertEqual(bn.slow.countnans(a), 2)

    def test1d_w(self):
        a = np.ones(5)
        a[2] = a[4] = float("nan")
        self.assertAlmostEqual(bn.countnans(a, weights=[0.1, 0.2, 0.3, 0.4, 0.5]), 0.8)
        self.assertAlmostEqual(bn.slow.countnans(a, weights=[0.1, 0.2, 0.3, 0.4, 0.5]), 0.8)



"nanmax template"

from copy import deepcopy
import bottlechest as bn

__all__ = ["nanmax"]

FLOAT_DTYPES = [x for x in bn.dtypes if 'float' in x]
INT_DTYPES = [x for x in bn.dtypes if 'int' in x]

# Float dtypes (not axis=None) ----------------------------------------------

floats = {}
floats['dtypes'] = FLOAT_DTYPES
floats['axisNone'] = False
floats['force_output_dtype'] = False
floats['reuse_non_nan_func'] = False

floats['top'] = """
@cython.boundscheck(False)
@cython.wraparound(False)
def NAME_NDIMd_DTYPE_axisAXIS(np.ndarray[np.DTYPE_t, ndim=NDIM] a):
    "Maximum of NDIMd array with dtype=DTYPE along axis=AXIS ignoring NaNs."
    cdef int allnan = 1
    cdef np.DTYPE_t amax, ai
"""

loop = {}
loop[2] = """\
    if nINDEX1 == 0:
        msg = "numpy.nanmax raises on a.shape[axis]==0; so Bottleneck does." 
        raise ValueError(msg)
    for iINDEX0 in range(nINDEX0):
        amax = MINDTYPE
        allnan = 1
        for iINDEX1 in range(nINDEX1):
            ai = a[INDEXALL]
            if ai >= amax:
                amax = ai
                allnan = 0
        if allnan == 0:       
            y[INDEXPOP] = amax
        else:
            y[INDEXPOP] = NAN
    return y
"""

floats['loop'] = loop

# Float dtypes (axis=None) --------------------------------------------------

floats_None = deepcopy(floats)
floats_None['axisNone'] = True

loop = {}
loop[1] = """\
    if nINDEX0 == 0:
        m = "numpy.nanmax raises on a.size==0 and axis=None; Bottleneck too." 
        raise ValueError(m)
    amax = MINDTYPE
    for iINDEX0 in range(nINDEX0):
        ai = a[INDEXALL]
        if ai >= amax:
            amax = ai
            allnan = 0
    if allnan == 0:       
        return np.DTYPE(amax)
    else:
        return np.DTYPE(NAN)
"""
loop[2] = """\
    if nINDEX0 * nINDEX1 == 0:
        m = "numpy.nanmax raises on a.size==0 and axis=None; Bottleneck too." 
        raise ValueError(m)
    amax = MINDTYPE
    for iINDEX0 in range(nINDEX0):
        for iINDEX1 in range(nINDEX1):
            ai = a[INDEXALL]
            if ai >= amax:
                amax = ai
                allnan = 0
    if allnan == 0:       
        return np.DTYPE(amax)
    else:
        return np.DTYPE(NAN)
"""

floats_None['loop'] = loop

# Int dtypes (not axis=None) ------------------------------------------------

ints = deepcopy(floats)
ints['dtypes'] = INT_DTYPES 

loop = {}
loop[2] = """\
    if nINDEX1 == 0:
        msg = "numpy.nanmax raises on a.shape[axis]==0; so Bottleneck does."
        raise ValueError(msg)
    for iINDEX0 in range(nINDEX0):
        amax = MINDTYPE
        for iINDEX1 in range(nINDEX1):
            ai = a[INDEXALL]
            if ai >= amax:
                amax = ai
        y[INDEXPOP] = amax
    return y
"""

ints['loop'] = loop

# Int dtypes (axis=None) ----------------------------------------------------

ints_None = deepcopy(ints) 
ints_None['axisNone'] = True

loop = {}
loop[1] = """\
    if nINDEX0 == 0:
        m = "numpy.nanmax raises on a.size==0 and axis=None; Bottleneck too."
        raise ValueError(m)
    amax = MINDTYPE
    for iINDEX0 in range(nINDEX0):
        ai = a[INDEXALL]
        if ai >= amax:
            amax = ai
    return np.DTYPE(amax)
"""
loop[2] = """\
    if nINDEX0 * nINDEX1 == 0:
        m = "numpy.nanmax raises on a.size==0 and axis=None; Bottleneck too."
        raise ValueError(m)
    amax = MINDTYPE
    for iINDEX0 in range(nINDEX0):
        for iINDEX1 in range(nINDEX1):
            ai = a[INDEXALL]
            if ai >= amax:
                amax = ai
    return np.DTYPE(amax)
"""

ints_None['loop'] = loop

# Slow, unaccelerated ndim/dtype --------------------------------------------

slow = {}
slow['name'] = "nanmax"
slow['signature'] = "arr"
slow['func'] = "bn.slow.nanmax(arr, axis=AXIS)"

# Template ------------------------------------------------------------------

nanmax = {}
nanmax['name'] = 'nanmax'
nanmax['is_reducing_function'] = True
nanmax['cdef_output'] = True
nanmax['slow'] = slow
nanmax['templates'] = {}
nanmax['templates']['float'] = floats
nanmax['templates']['float_None'] = floats_None
nanmax['templates']['int'] = ints
nanmax['templates']['int_None'] = ints_None
nanmax['pyx_file'] = 'func/%sbit/nanmax.pyx'

nanmax['main'] = '''"nanmax auto-generated from template"

def nanmax(arr, axis=None):
    """
    Maximum values along specified axis, ignoring NaNs.

    Parameters
    ----------
    arr : array_like
        Input array. If `arr` is not an array, a conversion is attempted.
    axis : {int, None}, optional
        Axis along which the maximum is computed. The default (axis=None) is
        to compute the maximum of the flattened array.

    Returns
    -------
    y : ndarray
        An array with the same shape as `arr`, with the specified axis removed.
        If `arr` is a 0-d array, or if axis is None, a scalar is returned.

    See also
    --------
    bottlechest.nanmin: Minimum along specified axis, ignoring NaNs.
    bottlechest.nanargmax: Indices of maximum values along axis, ignoring NaNs. 
    
    Examples
    --------
    >>> bn.nanmax(1)
    1
    >>> bn.nanmax([1])
    1
    >>> bn.nanmax([1, np.nan])
    1.0
    >>> a = np.array([[1, 4], [1, np.nan]])
    >>> bn.nanmax(a)
    4.0
    >>> bn.nanmax(a, axis=0)
    array([ 1.,  4.])
    
    """
    func, arr = nanmax_selector(arr, axis)
    return func(arr)

def nanmax_selector(arr, axis):
    """
    Return nanmax function and array that matches `arr` and `axis`.
    
    Under the hood Bottleneck uses a separate Cython function for each
    combination of ndim, dtype, and axis. A lot of the overhead in bn.nanmax()
    is in checking that `axis` is within range, converting `arr` into an
    array (if it is not already an array), and selecting the function to use
    to calculate the maximum.

    You can get rid of the overhead by doing all this before you, for example,
    enter an inner loop, by using the this function.

    Parameters
    ----------
    arr : array_like
        Input array. If `arr` is not an array, a conversion is attempted.
    axis : {int, None}
        Axis along which the maximum is to be computed.
    
    Returns
    -------
    func : function
        The nanamx function that matches the number of dimensions and
        dtype of the input array and the axis along which you wish to find
        the maximum.
    a : ndarray
        If the input array `arr` is not a ndarray, then `a` will contain the
        result of converting `arr` into a ndarray.

    Examples
    --------
    Create a numpy array:

    >>> arr = np.array([1.0, 2.0, 3.0])
    
    Obtain the function needed to determine the maximum of `arr` along
    axis=0:

    >>> func, a = bn.func.nanmax_selector(arr, axis=0)
    >>> func
    <function nanmax_1d_float64_axis0>
    
    Use the returned function and array to determine the maximum:
    
    >>> func(a)
    3.0

    """
    cdef np.ndarray a
    if type(arr) is np.ndarray:
        a = arr
    else:    
        a = np.array(arr, copy=False)
    cdef int ndim = PyArray_NDIM(a)
    cdef int dtype = PyArray_TYPE(a)
    if (axis is not None) and (axis < 0):
        axis += ndim
    cdef tuple key = (ndim, dtype, axis)
    try:
        func = nanmax_dict[key]
    except KeyError:
        if axis is not None:
            if (axis < 0) or (axis >= ndim):
                raise ValueError("axis(=%d) out of bounds" % axis)
        try:
            func = nanmax_slow_dict[axis]
        except KeyError:
            tup = (str(ndim), str(a.dtype), str(axis))
            raise TypeError("Unsupported ndim/dtype/axis (%s/%s/%s)." % tup)
    return func, a
'''



"nanmin template"

from copy import deepcopy
import bottlechest as bn

__all__ = ["nanmin"]

FLOAT_DTYPES = [x for x in bn.dtypes if 'float' in x]
INT_DTYPES = [x for x in bn.dtypes if 'int' in x]

# Float dtypes (not axis=None) ----------------------------------------------

floats = {}
floats['dtypes'] = FLOAT_DTYPES
floats['axisNone'] = False
floats['force_output_dtype'] = False
floats['reuse_non_nan_func'] = False

floats['top'] = """
@cython.boundscheck(False)
@cython.wraparound(False)
def NAME_NDIMd_DTYPE_axisAXIS(np.ndarray[np.DTYPE_t, ndim=NDIM] a):
    "Minimum of NDIMd array with dtype=DTYPE along axis=AXIS ignoring NaNs."
    cdef int allnan = 1
    cdef np.DTYPE_t amin, ai
"""

loop = {}
loop[2] = """\
    if nINDEX1 == 0:
        msg = "numpy.nanmin raises on a.shape[axis]==0; so Bottleneck does."
        raise ValueError(msg)
    for iINDEX0 in range(nINDEX0):
        amin = MAXDTYPE
        allnan = 1
        for iINDEX1 in range(nINDEX1):
            ai = a[INDEXALL]
            if ai <= amin:
                amin = ai
                allnan = 0
        if allnan == 0:       
            y[INDEXPOP] = amin
        else:
            y[INDEXPOP] = NAN
    return y
"""

floats['loop'] = loop

# Float dtypes (axis=None) --------------------------------------------------

floats_None = deepcopy(floats)
floats_None['axisNone'] = True

loop = {}
loop[1] = """\
    if nINDEX0 == 0:
        m = "numpy.nanmin raises on a.size==0 and axis=None; Bottleneck too." 
        raise ValueError(m)
    amin = MAXDTYPE
    for iINDEX0 in range(nINDEX0):
        ai = a[INDEXALL]
        if ai <= amin:
            amin = ai
            allnan = 0
    if allnan == 0:       
        return np.DTYPE(amin)
    else:
        return np.DTYPE(NAN)
"""
loop[2] = """\
    if nINDEX0 * nINDEX1 == 0:
        m = "numpy.nanmin raises on a.size==0 and axis=None; Bottleneck too." 
        raise ValueError(m)
    amin = MAXDTYPE
    for iINDEX0 in range(nINDEX0):
        for iINDEX1 in range(nINDEX1):
            ai = a[INDEXALL]
            if ai <= amin:
                amin = ai
                allnan = 0
    if allnan == 0:       
        return np.DTYPE(amin)
    else:
        return np.DTYPE(NAN)
"""

floats_None['loop'] = loop

# Int dtypes (not axis=None) ------------------------------------------------

ints = deepcopy(floats)
ints['dtypes'] = INT_DTYPES 

loop = {}
loop[2] = """\
    if nINDEX1 == 0:
        msg = "numpy.nanmin raises on a.shape[axis]==0; so Bottleneck does."
        raise ValueError(msg)
    for iINDEX0 in range(nINDEX0):
        amin = MAXDTYPE
        for iINDEX1 in range(nINDEX1):
            ai = a[INDEXALL]
            if ai <= amin:
                amin = ai
        y[INDEXPOP] = amin
    return y
"""

ints['loop'] = loop

# Int dtypes (axis=None) ----------------------------------------------------

ints_None = deepcopy(ints) 
ints_None['axisNone'] = True

loop = {}
loop[1] = """\
    if nINDEX0 == 0:
        m = "numpy.nanmin raises on a.size==0 and axis=None; Bottleneck too."
        raise ValueError(m)
    amin = MAXDTYPE
    for iINDEX0 in range(nINDEX0):
        ai = a[INDEXALL]
        if ai <= amin:
            amin = ai
    return np.DTYPE(amin)
"""
loop[2] = """\
    if nINDEX0 * nINDEX1 == 0:
        m = "numpy.nanmin raises on a.size==0 and axis=None; Bottleneck too."
        raise ValueError(m)
    amin = MAXDTYPE
    for iINDEX0 in range(nINDEX0):
        for iINDEX1 in range(nINDEX1):
            ai = a[INDEXALL]
            if ai <= amin:
                amin = ai
    return np.DTYPE(amin)
"""

ints_None['loop'] = loop

# Slow, unaccelerated ndim/dtype --------------------------------------------

slow = {}
slow['name'] = "nanmin"
slow['signature'] = "arr"
slow['func'] = "bn.slow.nanmin(arr, axis=AXIS)"

# Template ------------------------------------------------------------------

nanmin = {}
nanmin['name'] = 'nanmin'
nanmin['is_reducing_function'] = True
nanmin['cdef_output'] = True
nanmin['slow'] = slow
nanmin['templates'] = {}
nanmin['templates']['float'] = floats
nanmin['templates']['float_None'] = floats_None
nanmin['templates']['int'] = ints
nanmin['templates']['int_None'] = ints_None
nanmin['pyx_file'] = 'func/%sbit/nanmin.pyx'

nanmin['main'] = '''"nanmin auto-generated from template"

def nanmin(arr, axis=None):
    """
    Minimum values along specified axis, ignoring NaNs.

    Parameters
    ----------
    arr : array_like
        Input array. If `arr` is not an array, a conversion is attempted.
    axis : {int, None}, optional
        Axis along which the minimum is computed. The default (axis=None) is
        to compute the minimum of the flattened array.

    Returns
    -------
    y : ndarray
        An array with the same shape as `arr`, with the specified axis removed.
        If `arr` is a 0-d array, or if axis is None, a scalar is returned.

    See also
    --------
    bottlechest.nanmax: Maximum along specified axis, ignoring NaNs.
    bottlechest.nanargmin: Indices of minimum values along axis, ignoring NaNs. 

    Examples
    --------
    >>> bn.nanmin(1)
    1
    >>> bn.nanmin([1])
    1
    >>> bn.nanmin([1, np.nan])
    1.0
    >>> a = np.array([[1, 4], [1, np.nan]])
    >>> bn.nanmin(a)
    1.0
    >>> bn.nanmin(a, axis=0)
    array([ 1.,  4.])
    
    """
    func, arr = nanmin_selector(arr, axis)
    return func(arr)

def nanmin_selector(arr, axis):
    """
    Return nanmin function and array that matches `arr` and `axis`.
    
    Under the hood Bottleneck uses a separate Cython function for each
    combination of ndim, dtype, and axis. A lot of the overhead in bn.nanmin()
    is in checking that `axis` is within range, converting `arr` into an
    array (if it is not already an array), and selecting the function to use
    to calculate the minimum.

    You can get rid of the overhead by doing all this before you, for example,
    enter an inner loop, by using the this function.

    Parameters
    ----------
    arr : array_like
        Input array. If `arr` is not an array, a conversion is attempted.
    axis : {int, None}
        Axis along which the minimum is to be computed.
    
    Returns
    -------
    func : function
        The nanmin function that matches the number of dimensions and
        dtype of the input array and the axis along which you wish to find
        the minimum.
    a : ndarray
        If the input array `arr` is not a ndarray, then `a` will contain the
        result of converting `arr` into a ndarray.

    Examples
    --------
    Create a numpy array:

    >>> arr = np.array([1.0, 2.0, 3.0])
    
    Obtain the function needed to determine the minimum of `arr` along
    axis=0:

    >>> func, a = bn.func.nanmin_selector(arr, axis=0)
    >>> func
    <function nanmin_1d_float64_axis0>
    
    Use the returned function and array to determine the minimum:
    
    >>> func(a)
    1.0

    """
    cdef np.ndarray a
    if type(arr) is np.ndarray:
        a = arr
    else:    
        a = np.array(arr, copy=False)
    cdef int ndim = PyArray_NDIM(a)
    cdef int dtype = PyArray_TYPE(a)
    if (axis is not None) and (axis < 0):
        axis += ndim
    cdef tuple key = (ndim, dtype, axis)
    try:
        func = nanmin_dict[key]
    except KeyError:
        if axis is not None:
            if (axis < 0) or (axis >= ndim):
                raise ValueError("axis(=%d) out of bounds" % axis)
        try:
            func = nanmin_slow_dict[axis]
        except KeyError:
            tup = (str(ndim), str(a.dtype), str(axis))
            raise TypeError("Unsupported ndim/dtype/axis (%s/%s/%s)." % tup)
    return func, a
'''   
